use std::{collections::HashMap, fs::read_dir, path::PathBuf};

use raylib::prelude::*;

#[derive(Default)]
pub struct Textures {
	map: HashMap<String, Texture2D>,
}

impl Textures {
	pub fn load_dir(&mut self, folder: &str, rl: &mut RaylibHandle, thread: &RaylibThread) {
		for d in read_dir(folder).unwrap().flatten() {
			let path = d.path();
			if path.is_file() {
				let name = path.file_stem().unwrap().to_string_lossy();
				let texture = rl
					.load_texture(thread, &format!("{folder}/{name}.png"))
					.unwrap();
				self.map.insert(name.to_string(), texture);
			}
		}
	}

	pub fn get(&self, name: &str) -> &Texture2D {
		self.map
			.get(name)
			.unwrap_or_else(|| self.map.get("missing").unwrap())
	}
}

pub fn userdata_dir() -> PathBuf {
	PathBuf::from("user")
}

pub fn draw_scaled_texture(
	d: &mut RaylibDrawHandle,
	texture: &Texture2D,
	x: i32,
	y: i32,
	scale: f32,
) {
	let pos = Vector2::new(x as f32, y as f32);
	d.draw_texture_ex(texture, pos, 0., scale, Color::WHITE);
}

pub fn get_free_id<T>(items: &[T], id_fn: fn(&T) -> usize) -> usize {
	let mut id = 0;
	while items.iter().any(|i| id_fn(i) == id) {
		id += 1;
	}
	id
}
